/*
 * VERSION 1.0 Januar 7th 2010 LATE  File: 'api.c'
 * Author: Linda Fouedjio
*/

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/iocan128.h>
#include <avr/iocanxx.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>

#include "one_wire.h"
#include "one_wire_adc.h"
#include "one_wire_dualSwitch.h"
#include "one_wire_simpleSwitch.h"
#include "read_write_register.h"
#include "one_wire_temperature.h"

#include "adc.h"
#include "api.h"
#include "api_define.h"
#include "api_global.h"
#include "can.h"
#include "mem-check.h"


/* pointer of array for defined serial error number*/
const char se00[] PROGMEM = "no valid command name";
const char se01[] PROGMEM = "ID is too long";
const char se02[] PROGMEM = "mask is too long";
const char se03[] PROGMEM = "rtr is too long";
const char se04[] PROGMEM = "length is  too long";
const char se05[] PROGMEM = "data 0 is too long";
const char se06[] PROGMEM = "data 1 is too long";
const char se07[] PROGMEM = "data 2 is too long";
const char se08[] PROGMEM = "data is too long";
const char se09[] PROGMEM = "data 4 is too long";
const char se10[] PROGMEM = "data 5 is too long";
const char se11[] PROGMEM = "data 6 is too long";
const char se12[] PROGMEM = "data 7 is too long";
const char se13[] PROGMEM = "command is too long";
const char se14[] PROGMEM = "ID hat invalid type";
const char se15[] PROGMEM = "mask has invalid type";
const char se16[] PROGMEM = "rtr has invalid type";
const char se17[] PROGMEM = "length had invalid type";
const char se18[] PROGMEM = "data 0 had invalid type";
const char se19[] PROGMEM = "data 1 had invalid type";
const char se20[] PROGMEM = "data 2 had invalid type";
const char se21[] PROGMEM = "data 3 had invalid type";
const char se22[] PROGMEM = "data 4 had invalid type";
const char se23[] PROGMEM = "data 5 had invalid type";
const char se24[] PROGMEM = "data 6 had invalid type";
const char se25[] PROGMEM = "data 7 had invalid type";
const char se26[] PROGMEM = "undefined error type";
const char se27[] PROGMEM = "first value is too long";
const char se28[] PROGMEM = "second value is too long";

const char *serial_error[] = { se00, se01, se02, se03, se04, se05, se06, se07, se08, se09, se10, se11, se12, se13, se14, se15, se16, se17, se18, se19, se20, se21, se22, se23, se24, se25, se26, se27, se28 };


/* array for defined serial error number*/
const uint8_t serial_error_number[] ={ 11, 12, 13, 14, 15, 16, 17, 18, 19, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128,129
                                            };

/* pointer of array for defined can error number*/

const char ce00[] PROGMEM = "Can_Bus is off";
const char ce01[] PROGMEM = "Can_Bus is passive";
const char ce02[] PROGMEM = "Can_Bus is on";
const char ce03[] PROGMEM = "Bit Error";
const char ce04[] PROGMEM = "Stuff Error";
const char ce05[] PROGMEM = "CRC Error";
const char ce06[] PROGMEM = "Form Error";
const char ce07[] PROGMEM = "Acknowdlegment Error";
const char ce08[] PROGMEM = "CAN was not successfully initialized";
const char ce09[] PROGMEM = "timeout for CAN communication";

const char *can_error[] = { ce00, ce01, ce02, ce03, ce04, ce05, ce06, ce07, ce08, ce09 };


/* array for defined can error number*/
const uint8_t can_error_number[] ={ 21, 22, 23, 24, 25, 26, 27, 28, 29, 210
                                          };
/* pointer of array for defined mailbox error */

const char me00[] PROGMEM = "all mailbox already in use";
const char me01[] PROGMEM = "message ID not found";
const char me02[] PROGMEM = "this message already exists";

const char *mob_error[] = {
me00, me01, me02 };

/* array for defined mailbox error number*/
const uint8_t mob_error_number[] ={ 31, 32, 33  };


/* pointer of array for defined general error number*/

const char ge00[] PROGMEM = "init for timer0 failed";
const char ge01[] PROGMEM = "init for timer0A failed";
const char ge02[] PROGMEM = "family code not found";
const char ge03[] PROGMEM = "all message were read";
const char ge04[] PROGMEM = "no device is connected to the bus" ;
const char ge05[] PROGMEM = "undefined bus";
const char ge06[] PROGMEM = "channel undefined";
const char ge07[] PROGMEM = "value has invalid type";
const char ge08[] PROGMEM = "adress has invalid type";
const char ge09[] PROGMEM = "undefined family code";
const char ge10[] PROGMEM = "invalid argument";


/*enum ge {
	GE_init_for_timer0_failed=0,
	GE_init_for_timer0A_failed,
	GE_family_code_not_found,
	GE_all_message_were_read,
	GE_no_device_is_connected_to_the_bus=4,
	GE_undefined_bus,
	GE_channel_undefined,
	GE_value_has_invalid_type,
	GE_adress_has_invalid_type,
	GE_undefined_family_code,
	GE_invalid_argument
};
*/

const char *general_error[] = { ge00, ge01, ge02, ge03, ge04, ge05, ge06, ge07, ge08, ge09, ge10};

/* array for defined general error number*/
const uint8_t general_error_number[] = { 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 410};


/*
 *this function initializes the UART0 register needed to allow the at90can128
 *USART transmissions
 */
int8_t UART0_Init (void){
		uint8_t intstate1 = SREG;/*save global interrupt flag*/
		cli();/*disable interrupt*/
		while(0<<TXC0){
		}
		/* set baud rate*/
		UBRR0H = (uint8_t)(BAUD>>8);
		UBRR0L = (uint8_t)(BAUD);
		/* set frame format: i wil set it now as 8 bits, no parity 1 stop bit, asynchron*/
		UCSR0C = (0<<UMSEL0)|(0<<UPM0)|(0<<USBS0)|(3<<UCSZ0);
		/*set double speed*/
		UCSR0A =(1<<U2X0);
		/*enable receiver and transmitter; enable RX and Data register empty interrupt*/
		UCSR0B = (1<<TXEN0)|(1<<RXEN0)|(1<<RXCIE0);
		SREG=intstate1; /*restore global interrupt flag*/
		return 1;
}//END of  UART0_Init


/*
 *this function initializes the CAN register for AT90CAN128 and enable the CAN-controller
 * the function has not input variable
 * the return value is an integer:
 * 1  -> the CAN initializierung is successfully
 * -1 -> the CAN initializierung is unsuccessfully
*/
int8_t CAN_Init(int32_t Baudrate)
	{
		uint8_t intstate2 = SREG;/*save global interrupt flag*/
		cli();/*disable interrupt*/
		CANGCON |= (1<<SWRES);
		CANGCON &= ~(1<<SWRES);
		/*enable general interrupt interrupt*/
		CANGIE= (1<<ENIT) | (1<<ENBOFF) |(1<<ENRX) |(1<<ENTX) |(1<<ENERR)|(1<<ENBX)|(1<<ENERG)|(0<<ENOVRT);
		/* enable Interrupt of MOBs register*/
		CANIE2 = 0xFF;
		CANIE1 = 0x7F;

		for (uint8_t mob = 0; mob < 15; mob++)
		{
			CANPAGE  = (mob << MOBNB0); /* clear all mailbox*/
			CANSTMOB = 0x00;  /*clear  MOB status register*/
			CANCDMOB = 0x00;  /*clear MOB control*/
			CANGSTA = 0x00;   /*clear CAN general status register*/
			/*clear all identifier tag register*/
			CANIDT1 = 0X00;
			CANIDT2 = 0X00;
			CANIDT3 = 0X00;
			CANIDT4 = 0X00;
			/*clear all identifier mask register*/
			CANIDM1 = 0X00;
			CANIDM2 = 0X00;
			CANIDM3 = 0X00;
			CANIDM4 = 0X00;
			/*clear all CAN message register*/
			for (unsigned char j=0; j<8; j++)
			{
				CANMSG = 0;
			}
		}
		/* set CAN baudrate register */
		if(TWOHUNDERTFIFTY_KPS  == Baudrate)
		{
			CANBT1 = (3<<BRP0);
			CANBT2 = (4<<PRS0) | (0<<SJW0);
			CANBT3 = (1<<PHS20) | (1<<PHS10);

			CANGCON |= (1<<ENASTB); /* set ENA/STB enable mode  */
			CANCDMOB |= (0<<CONMOB0)|(0<<CONMOB1); /*enable reception mode here so that information for CAN come automatically*/
			SREG=intstate2; /*restore global interrupt flag*/
			return 1;
		}

		if(ONEHUNDERTTWENTYFIVE_KPS == Baudrate)
		{

			CANBT1 = (1<<BRP0);
			CANBT2 = (1<<PRS0) | (0<<SJW0);
			CANBT3 = (2<<PHS20) | (13<<PHS10);

			CANGCON |= (1<<ENASTB); /* set ENA/STB enable mode  */
			CANCDMOB |= (0<<CONMOB0)|(0<<CONMOB1); /*enable reception mode here so that information for CAN come automatically*/
			SREG=intstate2; /*restore global interrupt flag*/
			return 1;
		}
		else
		{
			return -1;
		}

 }// END of CAN_Init function


/*
 *this function deletes some CAN register for AT90CAN128
 * the function has no input and output parameters
*/

void Init(void){
		for (uint8_t mob = 0; mob < 15; mob++)
		{
			CANPAGE  = (mob << MOBNB0); /* clear all  mailbox*/
			CANSTMOB = 0x00;  /*clear  MOB status register*/
			CANCDMOB = 0x00;  /*clear MOB control register*/
			CANGSTA = 0x00;   /*clear CAN general status register*/
			/*clear all identifier tag register*/
			CANIDT1 = 0X00;
			CANIDT2 = 0X00;
			CANIDT3 = 0X00;
			CANIDT4 = 0X00;
			/*clear all identifier mask register*/
			CANIDM1 = 0X00;
			CANIDM2 = 0X00;
			CANIDM3 = 0X00;
			CANIDM4 = 0X00;
			for (unsigned char j = 0; j < 8; j++)
			{
				CANMSG = 0; /*clear all CAN message register*/
			}
		}
}//END of Init function


/*
 *this function initializes the 8 bit counter
 * the function has not input parameter
*/

int8_t Timer0_Init(void){
		uint8_t intstate3 = SREG; /* save global interrupt flag */
		cli(); /* disable interrupt */
		TCCR0A = (1<<CS02)|(0<<CS01)|(1<<CS00);  /* set Prescaler 1024 */
		TIMSK0 |= (1<<TOIE0); /* Timer0 Overflow enable */
		SREG=intstate3; /* restore global interrupt flag */
		return 1;
}// END of Timer0_Init



/*
 *this function initializes the 8 bit counter with output compareA
 * the function has not input parameter
 * the output parameter is an integer
 * 1 -> the timer initializierung for timer 0 was successfully
*/

int8_t Timer0A_Init(void){
		uint8_t intstate4 = SREG; /* save global interrupt flag */
		cli(); /* disable interrupt */
		TCCR0A = (1<<FOC0A)|(1<<CS02)| (0<<CS01)|(1<<CS00);  /* set Prescaler 1024 */
		OCR0A = 255;
		TCNT0 = 20;
		TIMSK0 |= (1<<OCIE0A); /* enable compare output Interrupt */
		SREG=intstate4; /*restore global interrupt flag */
		return 1;
}//END of int8_t Timer0A_Init


/*
 *this function dismantled the string to receive various parameters
 *the function has a string via serial interface as input parameter and returns no parameter
*/

void Decrypt_Uart_String(void) {
		uint8_t  index_parameter;
		uint8_t  length_decryptString;

		length_decryptString = strlen(decrypt_uartString);
		if(length_decryptString > MAX_LENGTH_COMMAND) {

			uart_errorCode = Communication_Error(serial_error[13],serial_error_number[13], "ERRA");

		}
		else {
			const char *delims= " ";
			char *test = NULL;
			char *result = NULL; /* pointer initialisieren */
			result = strtok_r(decrypt_uartString,delims,&test);  /*search space in string */
			index_parameter = 0; /* pointer of setParameter*/

			while( result != NULL ) {
				strncpy(setParameter[index_parameter],result, 8);
				result = strtok_r( NULL, delims, &test );
				index_parameter++;

				if(MAX_PARAMETER <= index_parameter)
				{
					uart_errorCode = Communication_Error(serial_error[13],serial_error_number[13], "ERRA");
					return;
				}
			}
			
		/* call the function for the conversion Frameformet in Canformat*/
			Convert_UartFormat_to_CanFormat(setParameter);


		}
}//END of Decrypt_Uart_String function


/*
 *this function assigns the various parameters of the string to the structure and reviewed the range of *individual  parameters.
 *the  function has a two dimensional array from string via serial interface as input and returns no parameter
*/

void  Convert_UartFormat_to_CanFormat(char string[MAX_PARAMETER][MAX_LENGTH_PARAMETER]) {

		/* convert char in Hexadezimal*/

		ptr_uartStruct->Name = setParameter[0];
		ptr_uartStruct->Uart_Message_ID = (uint32_t) strtoul(setParameter[1], &ptr_setParameter[1],16);
		ptr_uartStruct->Uart_Mask       = (uint32_t) strtoul(setParameter[2], &ptr_setParameter[2],16);
		ptr_uartStruct->Uart_Rtr        = (uint8_t)  strtoul(setParameter[3], &ptr_setParameter[3],16);
		ptr_uartStruct->Uart_Length     = (uint8_t)  strtoul(setParameter[4], &ptr_setParameter[4],16);
		ptr_uartStruct->Uart_Data[0]    = (uint16_t) strtoul(setParameter[5], &ptr_setParameter[5],16);
		ptr_uartStruct->Uart_Data[1]    = (uint16_t) strtoul(setParameter[6], &ptr_setParameter[6],16);
		ptr_uartStruct->Uart_Data[2]    = (uint16_t) strtoul(setParameter[7], &ptr_setParameter[7],16);
		ptr_uartStruct->Uart_Data[3]    = (uint16_t) strtoul(setParameter[8], &ptr_setParameter[8],16);
		ptr_uartStruct->Uart_Data[4]    = (uint16_t) strtoul(setParameter[9], &ptr_setParameter[9],16);
		ptr_uartStruct->Uart_Data[5]    = (uint16_t) strtoul(setParameter[10],&ptr_setParameter[10],16);
		ptr_uartStruct->Uart_Data[6]    = (uint16_t) strtoul(setParameter[11],&ptr_setParameter[11],16);
		ptr_uartStruct->Uart_Data[7]    = (uint16_t) strtoul(setParameter[12],&ptr_setParameter[12],16);
		Check_Parameter(ptr_uartStruct); /*call the function Check_Parameter*/

} // END of function Convert_UartFormat_to_CanFormat

/* this function checks whether all the received parameters are valid
 * the function has a pointer of the serial structur as input and returns no parameter
*/

void Check_Parameter(struct uartStruct *ptr_uartStruct){

	/*check range of parameter*/
#warning why are Uart_Data[5..7] different ?
	if(((strcmp_P (ptr_uartStruct->Name, PSTR("SEND"))== 0)||
	                              (strcmp_P (ptr_uartStruct->Name,PSTR ("SUBS"))== 0)||
				      (strcmp_P (ptr_uartStruct->Name, PSTR("USUB"))== 0)||
				      (strcmp_P (ptr_uartStruct->Name, PSTR("STAT"))== 0)||
				      (strcmp_P (ptr_uartStruct->Name, PSTR("RGWR"))== 0)||
				      (strcmp_P (ptr_uartStruct->Name, PSTR("RGRE"))== 0)||
				      (strcmp_P (ptr_uartStruct->Name, PSTR("RADC"))== 0)||
                                      (strcmp_P (ptr_uartStruct->Name, PSTR("OWAD"))== 0)||
                                      (strcmp_P (ptr_uartStruct->Name, PSTR("OWDS"))== 0)||
                                      (strcmp_P (ptr_uartStruct->Name, PSTR("OWON"))== 0)||
                                      (strcmp_P (ptr_uartStruct->Name, PSTR("OWLS"))== 0)||
                                      (strcmp_P (ptr_uartStruct->Name, PSTR("OWSS"))== 0)||
                                      (strcmp_P (ptr_uartStruct->Name, PSTR("RSET"))== 0)||
				      (strcmp_P (ptr_uartStruct->Name, PSTR("PING"))== 0)||
				      (strcmp_P (ptr_uartStruct->Name, PSTR("OWTP")))== 0) &&
				      (((0x7FFFFFF)>= (ptr_uartStruct->Uart_Message_ID))  && (( 0 ==(*ptr_setParameter[1]))   || (0x20==(*ptr_setParameter[1])  )))&&
				      (((0x7FFFFFF)>=(ptr_uartStruct->Uart_Mask))         && (( 0 ==(*ptr_setParameter[2]))   || (0x20==(*ptr_setParameter[2])  )))&&
				      (((1)>=(ptr_uartStruct->Uart_Rtr))                  && (( 0 ==(*ptr_setParameter[3]))   || (0x20==(*ptr_setParameter[3])  )))&&
				      (((8)>= (ptr_uartStruct->Uart_Length))              && (( 0 ==(*ptr_setParameter[4]))   || (0x20==(*ptr_setParameter[4])  )))&&
				      (((0XFF)>=(ptr_uartStruct->Uart_Data[0]))           && (( 0 ==(*ptr_setParameter[5]))   || (0x20==(*ptr_setParameter[5])  )))&&
				      (((0XFF)>=(ptr_uartStruct->Uart_Data[1]))           && (( 0 ==(*ptr_setParameter[6]))   || (0x20==(*ptr_setParameter[6])  )))&&
				      (((0XFF)>=(ptr_uartStruct->Uart_Data[2]))           && (( 0 ==(*ptr_setParameter[7]))  || (0x20==(*ptr_setParameter[7])  )))&&
				      (((0XFF)>=(ptr_uartStruct->Uart_Data[3]))           && (( 0 ==(*ptr_setParameter[8]))   || (0x20==(*ptr_setParameter[8])  )))&&
				      (((0XFF)>=(ptr_uartStruct->Uart_Data[4]))           && (( 0 ==(*ptr_setParameter[9]))   || (0x20==(*ptr_setParameter[9])  )))&&
				      (((0XFF)>=((ptr_uartStruct->Uart_Data[5]))          && (( 0 ==(*ptr_setParameter[10])) || (0x20==(*ptr_setParameter[10])  )))&&
				      (((0XFF)>=(ptr_uartStruct->Uart_Data[6]))           && (( 0 ==(*ptr_setParameter[11])) || (0x20==(*ptr_setParameter[11]) )))&&
				      (((0XFF)>=(ptr_uartStruct->Uart_Data[7]))           && (( 0 ==(*ptr_setParameter[12])) || (0x20==(*ptr_setParameter[12]) ))))

	  ){
		Choose_Function(ptr_uartStruct);

		/*clear the variable setParameter*/
		for(uint8_t count_setParameter = 0; count_setParameter < 13 ;count_setParameter++)
		{
			setParameter[count_setParameter][0] = 0;
		}
			// return from current function
	}

	else {
	        Check_Error(ptr_uartStruct);
	}


}// END of Check_Parameter function


/*this function checks error for the received parameter
 * the function has a pointer of the serial structur as input and returns no parameter
*/

void Check_Error(struct uartStruct *ptr_uartStruct){


	if(  (strcmp_P (ptr_uartStruct->Name, PSTR("SEND"))!= 0) &&
		     (strcmp_P (ptr_uartStruct->Name, PSTR("SUBS"))!= 0) &&
		     (strcmp_P (ptr_uartStruct->Name, PSTR("USUB"))!= 0) &&
		     (strcmp_P (ptr_uartStruct->Name, PSTR("STAT"))!= 0) &&
		     (strcmp_P (ptr_uartStruct->Name, PSTR("RGWR"))!= 0) &&
		     (strcmp_P (ptr_uartStruct->Name, PSTR("RGRE"))!= 0) &&
		     (strcmp_P (ptr_uartStruct->Name, PSTR("RADC"))!= 0) &&
                     (strcmp_P (ptr_uartStruct->Name, PSTR("OWAD"))!= 0) &&
                     (strcmp_P (ptr_uartStruct->Name, PSTR("OWON"))!= 0) &&
                     (strcmp_P (ptr_uartStruct->Name, PSTR("OWDS"))!= 0) &&
                     (strcmp_P (ptr_uartStruct->Name, PSTR("OWSS"))!= 0) &&
                     (strcmp_P (ptr_uartStruct->Name, PSTR("OWLS"))!= 0) &&
                     (strcmp_P (ptr_uartStruct->Name, PSTR("RSET"))!= 0) &&
		     (strcmp_P (ptr_uartStruct->Name, PSTR("PING"))!= 0) &&
		     (strcmp_P (ptr_uartStruct->Name, PSTR("OWTP"))!= 0))
	{

		uart_errorCode = Communication_Error(serial_error[0],serial_error_number[0], "ERRA");
		setParameter[0][0] = '\0';
	}


	if((0x7FFFFFF) < ptr_uartStruct->Uart_Message_ID) {

		uart_errorCode = Communication_Error(serial_error[1],serial_error_number[1], "ERRA");
		setParameter[1][0] = '\0';
		*ptr_setParameter[1] = 0;
	}
	if(0 != (*ptr_setParameter[1]) ) {

		uart_errorCode = Communication_Error(serial_error[14],serial_error_number[14], "ERRA");
		*ptr_setParameter[1] = 0;
	}

	if((0x7FFFFFF) < ptr_uartStruct->Uart_Mask) {

		uart_errorCode = Communication_Error(serial_error[2],serial_error_number[2], "ERRA");
		setParameter[2][0] = '\0';
		*ptr_setParameter[2] = 0;
	}
	if(0 != (*ptr_setParameter[2]) ) {

		uart_errorCode = Communication_Error(serial_error[15],serial_error_number[15], "ERRA");
		*ptr_setParameter[2] = 0;
	}

	if((1) < ptr_uartStruct->Uart_Rtr) {

		uart_errorCode = Communication_Error(serial_error[3],serial_error_number[3], "ERRA");
		setParameter[3][0] = '\0';
		*ptr_setParameter[3] = 0;
	}
	if(0 != (*ptr_setParameter[3]) ) {

		uart_errorCode = Communication_Error(serial_error[16],serial_error_number[16], "ERRA");
		*ptr_setParameter[3] = 0;
	}
	if((8) < ptr_uartStruct->Uart_Length) {

		uart_errorCode = Communication_Error(serial_error[4],serial_error_number[4], "ERRA");
		setParameter[4][0] = '\0';
		*ptr_setParameter[4] = 0;
	}
	if(0 != (*ptr_setParameter[4]) ) {

		uart_errorCode = Communication_Error(serial_error[17],serial_error_number[17], "ERRA");

		*ptr_setParameter[4] = 0;
	}


	for (uint8_t i = 0; i < 8; i++){
		if((0XFF) < ptr_uartStruct->Uart_Data[i]) {


			uart_errorCode = Communication_Error(serial_error[i+5],serial_error_number[i+5], "ERRA");

			ptr_uartStruct->Uart_Data[i] = 0;
			setParameter[i+5][0] = '\0';
			*(ptr_setParameter[i+5]) = 0;
		}
	}
	for (uint8_t i = 5; i < 13; i++){
		if(0 != (*ptr_setParameter[i]) ) {
			uart_errorCode = Communication_Error(serial_error[i+13],serial_error_number[i+13], "ERRA");

			*ptr_setParameter[i+13] = 0;
		}

	}

}//END of Check_Error function


/*
 *this function calls another function in relation to the first parameter of the string
 *the function has a pointer of the serial structur as input and returns no parameter
*/

void Choose_Function(struct uartStruct *ptr_uartStruct){
		if ((strcmp_P (ptr_uartStruct->Name, PSTR("SEND")) == 0) && (ptr_uartStruct->Uart_Rtr == 0))

			Send_Message(ptr_uartStruct); /* call function with name  Send_Message */
		else if ((strcmp_P(ptr_uartStruct->Name,PSTR("SEND"))== 0)&& (ptr_uartStruct->Uart_Rtr == 1))

			Receive_Message(ptr_uartStruct); /* call function with name Receive_message */

		else if ((strcmp_P (ptr_uartStruct->Name, PSTR("SUBS")) == 0))
		{
			Subscribe_Message(ptr_uartStruct); /* call function with name Subscribe_Message */
		}

		else if ((strcmp_P (ptr_uartStruct->Name, PSTR("USUB")) == 0))
		{
			Unsubscribe_Message(ptr_uartStruct); /* call function with name Unsubscribe_Message */
		}
		else if ((strcmp_P (ptr_uartStruct->Name, PSTR("STAT")) == 0))
		{

			//readStatus(ptr_uartStruct); /* call function with name readTempSensor */
		}

		else if ((strcmp_P (ptr_uartStruct->Name, PSTR("OWTP")) == 0))
		{

			readTempSensor(ptr_uartStruct); /* call function with name readTempSensor */

		}
		else if ((strcmp_P (ptr_uartStruct->Name,PSTR( "RGWR")) == 0))
		{
			writeRegister(ptr_uartStruct); /* call function with name writeRegister  */

		}
		else if ((strcmp_P (ptr_uartStruct->Name, PSTR("RGRE")) == 0))
		{
			readRegister(ptr_uartStruct); /* call function with name  readRegister */

		}
		else if ((strcmp_P (ptr_uartStruct->Name, PSTR("RADC")) == 0))
		{
			read_ADC(ptr_uartStruct); /* call function with name  readRegister */

		}
                else if ((strcmp (ptr_uartStruct->Name, "OWAD") == 0))
		{

			read_WADC(ptr_uartStruct); /* call function with name  readRegister */

		}
               else if ((strcmp (ptr_uartStruct->Name, "OWON") == 0))
		{

			dualSwitches_ON(ptr_uartStruct); /* call function with name  readRegister */

		}
               else if ((strcmp (ptr_uartStruct->Name, "OWDS") == 0))
		{

			read_status_dualSwitches(ptr_uartStruct); /* call function with name  readRegister */

		}

               else if ((strcmp (ptr_uartStruct->Name, "RSET") == 0))
		{

			reset_Software(ptr_uartStruct); /* call function with name  readRegister */

		}
                else if ((strcmp (ptr_uartStruct->Name, "OWSS") == 0))
		{

			read_status_simpleSwitches(ptr_uartStruct); /* call function with name  readRegister */

		}
               else if ((strcmp (ptr_uartStruct->Name, "OWLS") == 0))
		{

			list_all_devices(ptr_uartStruct); /* call function with name  readRegister */

		}
	        else if ((strcmp (ptr_uartStruct->Name, "PING") == 0))
		{

			keep_alive(ptr_uartStruct); /* call function with name  readRegister */
		}
		else
		{

			uart_errorCode = Communication_Error(serial_error[0],serial_error_number[0], "ERRA");
		}

}//END of Choose function


/* interrupt for overflow with defined interrupt vector TIMER0_OVF_vect */

ISR(TIMER0_OVF_vect){

		static  uint8_t  counter0 = 0;  /* counter for timer0 */
		static  uint8_t  counter1 = 0;  /* counter for timer0 */
		counter0++;
		counter1++;

		if(counter0 == 10)  /* 10 overflow for timer0=0.5s */{

			timer0Ready = 1; /* mark, that we got an CAN_interrupt, to be handled by main */
			counter0 = 0;
		}
		if(counter1 == 20)  /* 10 overflow for timer1=1s */{

			timer1Ready = 1; /* mark, that we got an CAN_interrupt, to be handled by main */
			counter1 = 0;
		}
}//END of ISR(TIMER0_OVF_vect)



/* interrupt for output compare with defined interrupt vector SIG_OUTPUT_COMPARE0 */
ISR(SIG_OUTPUT_COMPARE0){

		static  uint8_t  counter0A = 0; /* counter for timer0 with output compareA */
		static  uint8_t  counter0AScheduler = 0; /* counter for timer0 with output compareA */
		counter0A++;
		counter0AScheduler++;

		if(counter0A == 9) /*9 overflow for timer0A=0.6s*/{
			timer0AReady = 1;  /* mark, that we got an CAN_interrupt, to be handled by main */
			counter0A = 0;
		}
		if(counter0AScheduler == 40) /*9 overflow for timer0A=2s*/{
			timer0ASchedulerReady = 1;  /* mark, that we got an CAN_interrupt, to be handled by main */
			counter0AScheduler = 0;
		}
}//END of ISR(SIG_OUTPUT_COMPARE0)



/* interrupt for receive  data via serial communication with defined interrupt vector SIG_UART0_RECV */
ISR (SIG_UART0_RECV){

		unsigned char c = UDR0;

		if( c == '\n' ) /* the string is complete? */	{

			uartString[nextCharPos] = '\0';
			uartReady = 1;  /* mark, that we got an CAN_interrupt, to be handled by main */
			nextCharPos = 0;
		}

		else {
			uartString[nextCharPos] = c;
			nextCharPos++;
			uartString[nextCharPos] = '\0';
		}
}//END of ISR (SIG_UART0_RECV)


/* interrupt for receive data via CAN communication with defined interrupt vector CANIT_vect */

ISR(CANIT_vect){
		uint8_t save_canpage = CANPAGE;

		mob = Get_Interrupt_of_Mob(CANSIT2 | (CANSIT1 << 8)); /*get the index of the MOb for which the interrupt occured*/
		CANPAGE = mob<<4; /* select concerned MOb page */

		if (CANSTMOB & (1<<TXOK)){       CANSTMOB &= ~(1<<TXOK); /* disable transmition mode*/
		CANCDMOB = (1<<CONMOB0); /* reset receive mode*/

		}

		else if (CANSTMOB & (1<<RXOK)){
			ptr_canStruct->Can_Length = (CANCDMOB & (0xF<<DLC0));
			for ( uint8_t i=0; i< ptr_canStruct->Can_Length; i++)
			{	 /* get data of selected MOb */
				ptr_canStruct->Can_Data[i]= CANMSG ;
			}
			/*get identifier */
			ptr_canStruct->Can_Message_ID = 0;
			ptr_canStruct->Can_Message_ID = CANIDT2>>5;
			ptr_canStruct->Can_Message_ID |= CANIDT1<<3;
			ptr_canStruct->Can_Mob = mob; /*get mailbox */
			CANSTMOB &= ~(1<<RXOK); /* clear interrupt */
			CANCDMOB = (1<<CONMOB1); /* reset receive mode*/
			canReady = 1; /* mark, that we got an CAN_interrupt, to be handled by main */
		}
		CANPAGE =  save_canpage;   /* restore CANPAGE*/
}//END of ISR(CANIT_vect)




/* this function gets index of MOb for which an interrupt was detected
 * the function has an uint32_t CANSIT register as input parameter
 * the function has unsigned integer as input:
 * mob -> valid message object block
 * NOMOB -> no valid  message object block
*/

uint8_t Get_Interrupt_of_Mob( uint32_t bitmask) {

		if( bitmask == 0) return NOMOB;

 /*the bitmask is shifted to the rigth until the LSB is one
		the number of such shift operations is the index of the MOb*/
		for( mob = 0; (bitmask & 0x01) == 0; bitmask >>= 1, mob++);

		/*security check*/
		if ( mob > 14 ) return NOMOB;
      		else return mob;
} //END of Get_Interrupt_of_Mob function


/*
*this function gives the various CAN error
*the function has no input and output parameters
*/
void Get_CanError(void){


		if (CANSTMOB & (1<<BERR)){

			CANSTMOB &= ~(1<<BERR);
			can_errorCode = Communication_Error(can_error[3],can_error_number[3], "ERRC");
		}

		if (CANSTMOB & (1<<SERR)){

			CANSTMOB &= ~(1<<SERR);
			can_errorCode = Communication_Error(can_error[4],can_error_number[4], "ERRC");
		}

		if (CANSTMOB & (1<<CERR)){

			CANSTMOB &= ~(1<<CERR);
			can_errorCode = Communication_Error(can_error[5],can_error_number[5], "ERRC");
		}

		if (CANSTMOB & (1<<FERR)){

			CANSTMOB &= ~(1<<FERR);
			can_errorCode = Communication_Error(can_error[6],can_error_number[6], "ERRC");
		}

		if (CANSTMOB & (1<<AERR))
		{
			CANSTMOB &= ~(1<<AERR);
			can_errorCode = Communication_Error(can_error[7],can_error_number[7], "ERRC");

		}
}//END of Get_CanError


/*
 *this function gives the various CAN error on the channel
 *the function has no input and output parameters
*/


void Get_BusState(void){

		if (CANGSTA & (1<<BOFF)){

			can_errorCode = Communication_Error(can_error[0],can_error_number[0], "ERRC");
			CAN_Init(TWOHUNDERTFIFTY_KPS); /* CAN initialisieren */
		}

		if (CANGSTA & (1<<ERRP)){

			can_errorCode = Communication_Error(can_error[1],can_error_number[1], "ERRC");
			CAN_Init(TWOHUNDERTFIFTY_KPS); /* CAN initialisieren */

		}
}//END of Get_BusState function


/* this function gets the free communication channel
 * the function has no input parameter
 * and returns a integer value
 * freemob -> valid message object block
 * -1 -> no valid message object block
*/

int8_t Get_FreeMob(void){
		uint8_t ctrlReg;

		for ( uint8_t freemob = 2; freemob <= 14; freemob++){

			CANPAGE = freemob << 4;
			ctrlReg = (CANCDMOB & ((1<<CONMOB0) |(1<<CONMOB1)));
			if (ctrlReg == 0){

				return freemob;
			}
		}
		return -1;

}//END of Get_FreeMob function

/*
*USART0 must be initialized before using this function
*for use this function, be sure that data is no longer then 8 bit
*(single character) this function will send  8 bits from the at90can128
*/

void UART0_Transmit(uint8_t data){

		/* wait for empty transmit buffer */
		while(!(UCSR0A & (1<<UDRE0)))
		{
			/* do nothing*/
		}
		/* put the data into the buffer */

		UDR0 = data;
}
/*
 *this function sends a string to serial communication
 * the input parameter is a defined global vaviable tmp_str
 * the output parameter is an integer
 * 0 -> the message is sent
*/

int8_t UART0_Send_Message_String(char *tmp_str){
		for ( uint16_t j = 0; j < strlen ((char *) tmp_str) && j <  BUFFER_SIZE; j++){
			UART0_Transmit (tmp_str[j]);
		}
		UART0_Transmit ('\n');
		memset(tmp_str,0,sizeof(tmp_str)); /*clear tmp_str variable*/
		return 0;
}//END of UART0_Send_Message_String function

/*
 *this function sends a string to serial communication
 * the input parameter is a defined global vaviable tmp_str
 * the output parameter is an integer
 * 0 -> the message is sent
 * only use for the function FindFamilyDevicesAndGetValues in one_wire.c
*/
int8_t UART0_Send_Message_String_woLF(char *tmp_str){
		for ( uint16_t j = 0; j < strlen ((char *) tmp_str) && j <  BUFFER_SIZE; j++){
			UART0_Transmit (tmp_str[j]);
		}

		memset(tmp_str,0,sizeof(tmp_str)); /*clear tmp_str variable*/
		return 0;
}//END of UART0_Send_Message_String_woLF


/*
 *this function indicates an error for the serial,CAN and Mailbox communication
 * the function has as input the description of error and the error number
 * the output is an unsigned integer
 * 0 -> the error is  signaled
 * 1 -> an error has occured
 */


uint8_t Communication_Error( const char *Error, const uint8_t Error_Number,const char *Error_typ){

		char l_Error[BUFFER_SIZE]; /*to store the flash data*/
		strncpy_P(l_Error, Error, BUFFER_SIZE-1);

		if(strlen(l_Error)> MAX_LENGTH_ERROR){
			snprintf_P(tempstr,(BUFFER_SIZE-1),PSTR("ERRA %d %s"),Error_Number, l_Error);
			UART0_Send_Message_String(tempstr);
			return 0;
		}


		snprintf_P(tempstr,(BUFFER_SIZE-1),PSTR("%s %d %s"), Error_typ, Error_Number, l_Error);
		UART0_Send_Message_String(tempstr);



return  1;
}//END of Communication_Error function

/*
this function initialize all init functions again and actives the interrupt
*/
void reset_Software (struct uartStruct *ptr_uartStruct) {

        cli();  /* disable interrupts*/
	UART0_Init();
	Init_Port ();
        can_init = CAN_Init(TWOHUNDERTFIFTY_KPS);
        sei(); /* enable interrupts*/

}//END of reset_Software

/*this function checks the functionality of the software*/
void keep_alive (struct uartStruct *ptr_uartStruct)
{
		flag_pingActive = (0 != ptr_uartStruct->Uart_Message_ID);

        snprintf(tempstr,BUFFER_SIZE-1,"RECV PING mechanism is %s",(flag_pingActive)?"enabled":"disabled");
        UART0_Send_Message_String(tempstr);
}//END of keep_alive

