
/* The canapi.h is a header file, the functions and structure of the variables specific to the serial interface between the CPU and micro controller.
*the baudrate is 115200bit/s
*/
#ifndef API__H
#define API__H
#include "can.h"
#include "api_define.h"
#include "api_global.h"


/* Structure for CPU commands */
struct uartStruct
{
	char    *Name;
	uint32_t Uart_Message_ID;     
	uint32_t Uart_Mask;     
	uint8_t  Uart_Rtr;                
	uint8_t  Uart_Length; 
	uint16_t Uart_Data[8]; 
     
} ;
 extern struct uartStruct uartFrame;
 extern struct uartStruct *ptr_uartStruct;

/* Implemented functions */
void Check_Error(struct uartStruct *ptr_uartStruct); /*this function checks error for the received parameter */

void Check_Parameter(struct uartStruct *ptr_uartStruct);/* this function checks whether all the received parameters are valid*/

void Choose_Function(struct uartStruct *ptr_uartStruct);/* in terms of the command name is the competent function */

uint8_t Communication_Error( const char *Error, const uint8_t Error_Number,const char *Error_typ);/* error communication  for serial, CAN and mailbox*/

void Convert_pack_canFrame_to_UartFormat(struct canStruct *ptr_canStruct);/* this function collects the various CAN data in a string */

void Convert_UartFormat_to_CanFormat( char string[MAX_PARAMETER][MAX_LENGTH_PARAMETER]); /*converting the decomposed CPU format in    CAN-format */
//void Convert_UartFormat_to_CanFormat( char *string[MAX_LENGTH_PARAMETER]);


void keep_alive(struct uartStruct *PtrFrame); /*this function checks the functionality of the software*/


void Decrypt_Uart_String(void); /*CPU-cutting format in various parameters */

void Receive_Message(struct uartStruct *PtrFrame); /* function for the SEND command name and RTR set, initialization of the registers with elements of the structure uartStruct*/

void reset_Software(struct uartStruct *ptr_uartStruct); /*this function initialize all init functions again and actives the interrupt*/


void Send_Message(struct uartStruct *PtrFrame);/*function for the SEND command name and RTR is not set, initialization of the registers with elements of the structure uartStruct*/

void Subscribe_Message(struct uartStruct *PtrFrame); /* function for the command name SUBS, initialization of the registers with elements of the structure uartStruct */ 

int8_t Timer0_Init(void); /*initializieren  bit Timer0 */

int8_t Timer0A_Init(void); /* initializieren 8 bit Timermit output compareA*/

void Unsubscribe_Message(struct uartStruct *PtrFrame); /* function for the command name USUB, initialization of the registers with elements of the structure uartStruct */

int8_t UART0_Init (void); /* initializieren serial communication */

void UART0_Transmit(unsigned char c);/* function send  data direction to cpu */
 
int8_t UART0_Send_Message_String(char *tempstr);/* help function for the output of register contents in CPU */

int8_t UART0_Send_Message_String_woLF(char *tempstr);/* help function for the output of register contents in CPU */


#endif 
