/*
 * VERSION 1.0 Januar 7th 2010 LATE  File: 'one_wire_adc.c'
 * Author: Linda Fouedjio  based on Alejandro Gil
 */

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include <avr/iocan128.h>

#include "one_wire.h"
#include "one_wire_adc.h"
#include "one_wire_dualSwitch.h"
#include "one_wire_simpleSwitch.h"
#include "read_write_register.h"
#include "one_wire_temperature.h"

#include "adc.h"
#include "api.h"
#include "api_define.h"
#include "api_global.h"
#include "can.h"
#include "mem-check.h"

#include "OWIDeviceSpecific.h"
#include "OWIHighLevelFunctions.h"
#include "OWIBitFunctions.h"

/*
 * this function contains all the functions that are necessary to
 * list ID on all devices connected to one wire bus
 *the input parameter is a family code
 */
void list_all_devices(struct uartStruct *ptr_uartStruct) {

	NumDevicesFound = ReadDevicesID(BUSES, 1);
}//END of list_all_devices function

/*
 *this function list all ID of devices connected to bus
 */

int8_t ReadDevicesID(uint8_t *pins, int8_t showlist) {

	char foundDevices[NUM_DEVICES]; /*variable to store all fund devices*/
	memset(foundDevices, 0, sizeof(foundDevices));

	uint8_t countFoundFamilycode = 0;

	uint32_t familyCode = ptr_uartStruct->Uart_Message_ID;

	/*clean one IDS array*/
	for (int8_t a = 0; a < 7; a++) {
		IDS_ALL[NUM_DEVICES - 1][a] = 0;
	}

	for (int8_t b = 0; b < PIN_BUS + 1; b++) { /*initialize the bus*/

		OWI_Init(pins[b]); /*b Indicates the number of pins active for 1-wire bus on the HadCon board. If b is 0, it means pin 0 of the connector, if 1 pin 1 and so on*/
		uint8_t res = 1;
		countDEV = 0; /*reset number of devices */
		countDEVbus = 0;
		countDEV = countDEV + countDEVbus;

		if ((PD = OWI_DetectPresence(pins[b])) != 0) {
			/*this will search the devices Id�s till the max number of devices is reached or
			 *all the devices have been read out
			 */

			while (countDEVbus < NUM_DEVICES && res != OWI_ROM_SEARCH_FINISHED) {
				if (countDEVbus == 0) {
					res = 0;
				}

				OWI_DetectPresence(pins[b]);

				res = OWI_SearchRom(IDS_ALL[NUM_DEVICES - 1], res, pins[b]);

				for (int8_t a = 0; a < 8; a++) {
					IDS_ALL[countDEVbus][a] = IDS_ALL[NUM_DEVICES - 1][a];
				}
				countDEVbus++;
			}

			/*this loop will record the IDS on regtemp*/
			for (uint8_t q = 0; q < countDEVbus; q++) {
				// snprintf(tempstr,BUFFER_SIZE-1,"countDEV:%x",countDEV);
				//  			UART0_Send_Message_String(tempstr);
				switch (showlist) {
				case 0: // just return number of found devices
					return countDEVbus;
					break;
				case 1: // output device id of found devices
					// does requested Familiy Code (familyCode)
					// match the current one or are all allowed (0), else continue
					if (!((familyCode == 0) || (IDS_ALL[q][0] == familyCode))) {
						continue;
					}

					/*send the data (activate if DEBUG to see the list of all found devices*/
					snprintf(foundDevices, BUFFER_SIZE - 1,
							"%s%d: %02X%02X%02X%02X%02X%02X%02X%02X",
							(0 == countFoundFamilycode)?"RECV ":" ",
							countFoundFamilycode, IDS_ALL[q][0],IDS_ALL[q][1],
							IDS_ALL[q][2], IDS_ALL[q][3], IDS_ALL[q][4],
							IDS_ALL[q][5], IDS_ALL[q][6], IDS_ALL[q][7]);

					for (int8_t j = 0; j < strlen(foundDevices); j++) {
						if (foundDevices[j] != ('\n') || foundDevices[j]
								!= ('\0')) { /*only non terminating characters have to be sent*/
							UART0_Transmit(foundDevices[j]);
						} //end  of if
					} //end of j
					countFoundFamilycode++;
					break;
				default:
					general_errorCode = Communication_Error(general_error[10],
							general_error_number[10], "ERRG");
				}
			} //end of for q

			// showlist: found any matching device??
			if (1 == showlist && 0 == countFoundFamilycode && 0 != familyCode) {
				general_errorCode = Communication_Error(general_error[2],
						general_error_number[2], "ERRG");
			}//end of 'if ( 1 == showlist && 0 == strlen(foundDevices))'
		} //end of if((PD=OWI_DetectPresence(pins[b]))!=0)
		else {
			general_errorCode = Communication_Error(general_error[4],
					general_error_number[4], "ERRG");
		}
	}// end of OWI_Init(pins[b]);

	UART0_Transmit('\n');
	memset(foundDevices, 0, sizeof(foundDevices)); /*clear tempstr variable*/
	return countDEVbus;

}// END of ReadDevicesID function

/*
 *tthis function read the value for temperatur of sensor, ADC and switch
 */


int8_t FindFamilyDevicesAndGetValues(uint8_t *pins, int8_t countDev,uint8_t familyID) {


	char String_to_Send[BUFFER_SIZE];
        memset(String_to_Send, 0, BUFFER_SIZE);

	int8_t q = 0;

	uint16_t readValueADC = 0;/*variable to read value of ADC channel*/
	uint16_t readValueTemp = 0;/*variable to read temperatur of sensor*/
	uint8_t readValueDS = 0;/*variable to read value of dual switch*/
	uint8_t readValueSS = 0;/*variable to read value of simple switch*/

	if ((familyID == FAMILY_DS2450_ADC) || (familyID == FAMILY_DS18B20_TEMP)
			|| (familyID == FAMILY_DS2413_DUAL_SWITCH) || (familyID
			== FAMILY_DS2405_SIMPLE_SWITCH)) {

		if (0 == countDev) {

			general_errorCode = Communication_Error(general_error[4],
					general_error_number[4], "ERRG");
			return 0;

		}

		if (0 != countDev) {

			uint8_t foundCounter = 0;

			for (int8_t b = 0; b < PIN_BUS + 1; b++) {

				for (q = 0; q < countDev; q++) {

					if (familyID != IDS_ALL[q][0]) {
						continue;
					}

					switch (familyID) {
					case FAMILY_DS2450_ADC:

						readValueADC = ReadADCchannels(pins[b], IDS_ALL[q],
								IDS_ALL_VALUES[q], 4);

						if (1 != readValueADC) {
							snprintf(String_to_Send, BUFFER_SIZE - 1,
									"Error reading ADC channels: %d", q);
							UART0_Send_Message_String(tempstr);

							return 0;
						} else {
							snprintf(String_to_Send,
									BUFFER_SIZE - 1,
									"RECV %d: %02X%02X%02X%02X%02X%02X%02X%02X %.4X %.4X %.4X %.4X\r",
									foundCounter,
									IDS_ALL[q][0], IDS_ALL[q][1],
									IDS_ALL[q][2], IDS_ALL[q][3],
									IDS_ALL[q][4], IDS_ALL[q][5],
									IDS_ALL[q][6], IDS_ALL[q][7],
									IDS_ALL_VALUES[q][0], IDS_ALL_VALUES[q][1],
									IDS_ALL_VALUES[q][2], IDS_ALL_VALUES[q][3]);
						}
						break;
					case FAMILY_DS2413_DUAL_SWITCH:

						readValueDS = ReadDualSwitches(pins[b], IDS_ALL[q],
								&readValueDS);

						snprintf(String_to_Send, BUFFER_SIZE - 1,
								"RECV  %d: %02X%02X%02X%02X%02X%02X%02X%02X %.2X\r",
								foundCounter, IDS_ALL[q][0], IDS_ALL[q][1],
								IDS_ALL[q][2], IDS_ALL[q][3], IDS_ALL[q][4],
								IDS_ALL[q][5], IDS_ALL[q][6], IDS_ALL[q][7],
								readValueDS);
						break;

					case FAMILY_DS18B20_TEMP:

						readValueTemp = ReadTemperature(pins[b], IDS_ALL[q]);

						snprintf(String_to_Send, BUFFER_SIZE - 1,
								"RECV  %d: %02X%02X%02X%02X%02X%02X%02X%02X %.4X\r",
								foundCounter, IDS_ALL[q][0], IDS_ALL[q][1],
								IDS_ALL[q][2], IDS_ALL[q][3], IDS_ALL[q][4],
								IDS_ALL[q][5], IDS_ALL[q][6], IDS_ALL[q][7],
								readValueTemp);

						break;

					case FAMILY_DS2405_SIMPLE_SWITCH:

						readValueSS = ReadSimpleSwitches(pins[b], IDS_ALL[q],
								&readValueSS);

						snprintf(String_to_Send, BUFFER_SIZE - 1,
								"RECV %d: %02X%02X%02X%02X%02X%02X%02X%02X %.2X\r",
								foundCounter, IDS_ALL[q][0], IDS_ALL[q][1],
								IDS_ALL[q][2], IDS_ALL[q][3], IDS_ALL[q][4],
								IDS_ALL[q][5], IDS_ALL[q][6], IDS_ALL[q][7],
								readValueSS);

						break;

					default:
						general_errorCode = Communication_Error(
								general_error[9], general_error_number[9],
								"ERRG");
						return 0;
						break;
					}
					foundCounter++;

					/*send the data*/

					UART0_Send_Message_String_woLF(String_to_Send);

				}//for (q=0;q<NUM_DEVICES;q++)

			}//for(int b=0;b<PIN_BUS+1;b++)
		}//for if
	}//for if countDev


	else {
		general_errorCode = Communication_Error(general_error[9],
				general_error_number[9], "ERRG");

		return 0;

	}


	if (0 < strlen(String_to_Send)) {UART0_Transmit('\n');}
    memset(String_to_Send, 0, BUFFER_SIZE); /*clear String_to_Send variable*/

	return 1;
}//END of FindFamilyDevicesAndGetValues
