
/*
 * VERSION 1.0 Januar 7th 2010 LATE  File: 'one_wire_temperature.c'
 * Author: Linda Fouedjio  based on Michael Traxler based and Giacomo Ortona's hadtempsens
*/


#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/iocan128.h>


#include "api.h"
#include "api_define.h"
#include "api_global.h"
#include "can.h"
#include "mem-check.h"

#include "one_wire.h"
#include "one_wire_adc.h"
#include "one_wire_dualSwitch.h"
#include "read_write_register.h"
#include "one_wire_simpleSwitch.h"
#include "one_wire_temperature.h"

#include "OWIDeviceSpecific.h"
#include "OWIHighLevelFunctions.h"
#include "OWIBitFunctions.h"

#define DS1820_START_CONVERSION         0x44
#define DS1820_READ_SCRATCHPAD          0xbe
#define DS1820_WRITE_SCRATCHPAD         0x4E
#define DS1820_CONFIG_REGISTER          0x7f
#define DS1820_READ_POWER               0xb4


#define ADC_CYCLES 3 /*the number of ADC conversions done to get an average voltage value*/
#define ADC_TIMEOUT 1000 /*wait ADC_TIMEOUT clock cycles for a single ADC conversion*/


/*This function initializes all input /output of the microcontroller
 *the  function has no input and output variable
*/

 void Init_Port (void) {

	 uint8_t intstate = SREG; /* save global interrupt flag */
	 cli ();

	 /* use port A as output*/
	 DDRA = 0xFF;
	 PORTA = 0x00;
	 
	 /* use port B as input with activated pullups*/
	 DDRB = 0x00;
	 PORTB = 0xFF;

	 /* use port C as input with activated pullups*/
	 DDRC = 0xff & (~0x4);
       //  DDRC = 0xff ;
	 PORTC = 0xff;
	
	 /* use port E as input with activated pullups*/
	 DDRE = 0x00;
	 PORTE = 0xFF;

	 /* use port F as input with ctivated pullups*/
	 DDRF = 0x00;
	 PORTF = 0xFF;

	 /* enable ADC and set clock prescale factor to 64 (p.280)*/
	 ADCSRA = BIT (ADEN) | 0x06;
	 /* 0x80 => ADEN => enable ADC*/

	 /* low speed and manually triggered ADC conversion mode*/
	 ADCSRB = 0;

	 /* deactivate digital input buffer for used ADC pins*/
#ifdef DISABLE_JTAG
	 DIDR0 = 0xFF;
	 /*MCUCR |= 0x80;   // deactivate whole JTAG interface */
#else
	 DIDR0 = 0x0F;
#endif

	 /* enable only compare match interrupt on timer 1, channel A*/
	 TIMSK1 = 0x02;

	 /* normal counter mode (no wave and/or PWM stuff) for timer 1, channel A*/
	 TCCR1A = 0;
	 TCCR1C = 0;

	/* snprintf_P(tempstr,BUFFER_SIZE-1, PSTR("init finished") );
	 UART0_Send_Message_String(tempstr);*/	
	 
	 
	 SREG=intstate; /*restore global interrupt flag */

}//END of Init function				


/*
 *this function contains all the functions that are necessary to
 *read the temperatur of sensor  
 * as input parameters the function needs only a keyword 
 * the function has no return parameter
*/

void readTempSensor(struct uartStruct *ptr_uartStruct) {

                
		   OWI_ResetPulse(BUSES[0]);//very important
                   NumDevicesFound=ReadDevicesID(BUSES, 0);
		   if ( 0 != NumDevicesFound)
		   {
		      MakeTempConversionOfAllDevices(BUSES);
		      FindFamilyDevicesAndGetValues(BUSES, NumDevicesFound, FAMILY_DS18B20_TEMP);
		   }
// 		   else
// 		   {
// 		   general_errorCode = Communication_Error(general_error[4],general_error_number[4], "ERRG");
// 		   }
}
/*
*this function
*/
 
/* 
 *this function will read the data conteined in the scratchpad of the device 
 *identified by the id ID number
 *will return the temperature value stored in the scratchpad
 *if an error occurs, this function will return a temp lower then -900 degrees
*/

       
 uint16_t ReadTemperature(unsigned char bus, unsigned char * id) {

	   uint16_t temperature;
    
	   /* Reset, presence.*/
    
	   if (OWI_DetectPresence(bus)==0) {
		 return -999; // Error
	    }

	    /* Match id found earlier*/
	    OWI_MatchRom(id, bus);
	    /*Send READ SCRATCHPAD command.*/
	    OWI_SendByte(DS1820_READ_SCRATCHPAD, bus);
	   /* Read only two first bytes (temperature low, temperature high)*/
	    /* and place them in the 16 bit temperature variable.*/
	    temperature = OWI_ReceiveByte(bus); 
	    temperature |= (OWI_ReceiveByte(bus) << 8);

	    return temperature;

}// END of ReadTemperature function



/*
*this function
*/
 void toggle_pin (unsigned char pin_number){
	 
	 long int i, c, d;
	 _delay_ms (10);
	 
	 for (i = 0; i <= 50; i++) {
		 _delay_ms (10);
		 d = d + i * c;
		 c = d + i;
	 }
	 /*toggle FrontLED*/
	 PINA = 0x3;
        }//END of toggle_pin function


 enum{
	 eNoError = 0,
	 eWrongCANAddress = 1,		// bogus address set on hex-switches
	 eCANDataSize = 2,		// bogus amount of bytes arrived
	 eI2CTimeout = 4,		// I²C problem
	 eI2CFatalError = 8,		// I²C problem
	 eI2CRetryFailed = 16,		// I²C problem
	 eADCTimeout = 32,		// ADC conversion failed
	 eCPLDAcknowledgeTimeout = 64	// CPLD communication problem
 } ErrorCodes;

 
 
/*this function reads and converts the voltage of the ADC-4 channels
 * as input parameters the function needs number of channel to read a voltage
 * the value returned is the voltage read from the channel
*/

 int8_t GetVoltage (unsigned int channel_nr){
	 
	 uint16_t Timeout;
	 uint16_t Value;
	 uint16_t iStep;
	 int16_t TotalValue;


#ifndef DISABLE_JTAG
  /* we need the JTAG interface, thus do not measure
	 *the upper four ADC channels (used for JTAG, too)
  */
	 if (channel_nr > 3) {
		 Voltages[channel_nr] = 0;
		 return eNoError;
	 }
#endif

  /*use 3.3V supply voltage as reference voltage 
  * and select the voltage to be measured*/
	 ADMUX = 0x40 | (unsigned char) channel_nr;
	 /*ADMUX = 0x40 | 0x1B;*/

	 TotalValue = 0;
	 for (iStep = 0; iStep < ADC_CYCLES; iStep++) {
		 /* start a new single conversion*/
		 ADCSRA |= BIT (ADSC);

		 /* wait until conversion is complete*/
		 Timeout = ADC_TIMEOUT;
		 while (!(ADCSRA & BIT (ADIF)) && (--Timeout > 0));

		 /* clear interrupt flag from former conversion*/
		 ADCSRA |= BIT (ADIF);

		 if (Timeout == 0) {
			 return eADCTimeout;
		 }

		 /*one must read ADCL before ADCH!*/
		 Value = ADCL;
		 Value |= (unsigned int) (ADCH & 0x03) << 8;
		 TotalValue += Value;
	 }
	
	 TotalValue = TotalValue /ADC_CYCLES;
	
	 /*save results*/
	 Voltages[channel_nr] = (uint16_t) TotalValue;

	 return eNoError;
}// END of GetVoltage function



int MakeTempConversionOfAllDevices(uint8_t *pins) {



    PORTA |= 0x8;
     
           //OWI_MatchRom(id, bus);  // Match id founded earlier
       //OWI_SendByte(0x5A, bus); //PIO Access write command

    for(int b=0;b<PIN_BUS+1;b++) {
            //OWI_Init(pins[b]);
        if (OWI_DetectPresence(pins[b])==0){
            return -999; // Error
          }
       
        OWI_SendByte(OWI_ROM_SKIP, pins[b]);
        OWI_SendByte(DS1820_START_CONVERSION, pins[b]);
        while (OWI_ReadBit(pins[b])==0){
 
            //sprintf(cadena, "loop waiting for the conversion of all temperature sensors");
            //USART_send_string (cadena);
            _delay_us(100);
        }
    }
    return 1; //1=ok
}
